//  Copyright (C) 2012  Stephen Compall
// 
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
// 
//        http://www.apache.org/licenses/LICENSE-2.0
// 
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.

namespace Nocandy.LinqResources.FS.Mystery

type Mystery<'e, 'a> =
    | UnknownError of string
    | Failure of 'e
    | Success of 'a

    static member comp = MysteryBuilder ()

    /// Catamorphism.
    member m.Fold(uf, ff, sf) =
        match m with
        | UnknownError s -> uf s
        | Failure e -> ff e
        | Success a -> sf a

and MysteryBuilder() =
    /// Supports let!, do!.
    member z.Bind (m: Mystery<'e, 'a>, f): Mystery<'e, 'b> =
        m.Fold(UnknownError, Failure, f)

    /// Supports let, do.
    member z.Let (v, f): Mystery<'e, 'a> = f v

    /// Supports partial if.
    member z.Zero () = UnknownError "short-circuit"

    /// Supports return.
    member z.Return a = Success a

    /// Supports return!
    member z.ReturnFrom (fa: Mystery<'e, 'a>) = fa

    /// MonadPlus.
    member z.Combine (a: Mystery<'e, 'a>, b) =
        a.Fold((fun _ -> b), (fun _ -> b), (fun _ -> a))
