//  Copyright (C) 2012  Stephen Compall
// 
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
// 
//        http://www.apache.org/licenses/LICENSE-2.0
// 
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.

namespace Nocandy.LinqResources.FS.ReaderV

open Nocandy.LinqResources.FS.Mystery

type ReaderV<'r, 'e, 'a> =
    | ReaderV of ('r -> Mystery<'e, 'a>)

    static member comp = ReaderVBuilder ()

    member v.Run r =
        match v with ReaderV f -> f r

    member v.Contramap f =
        ReaderV (f >> v.Run)

    // member v.Compose(fst: ReaderV<'s, 'e, 'r>): ReaderV<'s, 'e, 'a> =
    //     ReaderV.comp { let! r = fst
    //                    return! ReaderV(fun _ -> v.Run r) }

and ReaderVBuilder() =
    /// Supports let!, do!.
    member z.Bind(m: ReaderV<'r, 'e, 'a>,
                  f: 'a -> ReaderV<'r, 'e, 'b>): ReaderV<'r, 'e, 'b> =
        ReaderV (fun r -> Mystery.comp { let! a = m.Run r
                                         return! (f a).Run r })

    /// Supports let, do.
    member z.Let(x, f) : ReaderV<'r, 'e, 'a> =
        z.Delay (fun () -> f x)

    /// Supports delayed computation.
    member z.Delay (ffa: unit -> ReaderV<'r, 'e, 'a>) : ReaderV<'r, 'e, 'a> =
        ReaderV (fun r -> (ffa ()).Run r)

    /// Supports partial if.
    member z.Zero () = ReaderV (fun _ -> Mystery.comp.Zero())

    /// Supports lazy fallback.
    member z.Combine (a: ReaderV<'r, 'e, 'a>, b: ReaderV<'r, 'e, 'a>) =
        ReaderV (fun r -> let ar = a.Run r
                          ar.Fold((fun _ -> b.Run r),
                                  (fun _ -> b.Run r),
                                  (fun _ -> ar)))

    /// Supports return.
    member z.Return x = ReaderV (fun _ -> Success x)

    /// Supports return!.
    member z.ReturnFrom (fa: ReaderV<'r, 'e, 'a>) = fa
