//  Copyright (C) 2012  Stephen Compall
// 
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
// 
//        http://www.apache.org/licenses/LICENSE-2.0
// 
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.
using System;
using NUnit.Framework;
using Nocandy.LinqResources.Mystery;

namespace Nocandy.LinqResources.Mystery.Test
{
  [TestFixture()]
  class TestMystery
  {
    [Test()]
    public void Equality ()
    {
      Assert.AreEqual (new UnknownError<int, int>("hi"),
                       new UnknownError<int, int>("hi"));
      Assert.AreEqual (new Failure<string, int>("broke"),
                       new Failure<string, int>("broke"));
      Assert.AreEqual (new Success<int, string>("fixed"),
                       new Success<int, string>("fixed"));
      Assert.That (new UnknownError<int, int>("greetings"),
                   Is.Not.EqualTo (new UnknownError<int, int>("hi")));
      Assert.That (new Failure<string, int>("broken"),
                   Is.Not.EqualTo (new Failure<string, int>("broke")));
      Assert.That (new Success<int, string>("perfect"),
                   Is.Not.EqualTo (new Success<int, string>("fixed")));
      Assert.That (new UnknownError<string, int>("hi"),
                   Is.Not.EqualTo (new Failure<string, int>("hi")));
      Assert.That (new Success<string, string>("hi"),
                   Is.Not.EqualTo (new Failure<string, string>("hi")));
      Assert.That (new Success<int, string>("hi"),
                   Is.Not.EqualTo (new UnknownError<int, string>("hi")));
    }

    private static Mystery<string, string> good =
      new Success<string, string>("good");
    private static Mystery<string, string> bad =
      new Failure<string, string>("bad");
    private static Mystery<string, string> what =
      new UnknownError<string, string>("what");

    [Test()]
    public void Mapping ()
    {
      Assert.That (from x in good select "super",
                   Is.EqualTo (new Success<string, string>("super")));
      Assert.That (from x in bad select "awesome",
                   Is.EqualTo (bad));
      Assert.That (from x in what select "who now",
                   Is.EqualTo (what));
    }

    [Test()]
    public void Identity ()
    {
      foreach (var act in new[] {good, bad, what})
      {
        Assert.AreEqual
          (act, from e in act from p in AMystery<string, string>.Point(e)
           select p);
        Assert.AreEqual
          (act, AMystery<string, string>.Join
           (AMystery<string, Mystery<string, string>>.Point (act)));
        Assert.AreEqual
          (act, AMystery<string, string>.Join
           (from a in act select AMystery<string, string>.Point (a)));
      }
    }

    [Test()]
    public void Flatmap ()
    {
      Assert.That (from x in good from y in good select x + y,
                   Is.EqualTo (new Success<string, string>("goodgood")));
    }
  }
}

