//  Copyright (C) 2012  Stephen Compall
// 
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
// 
//        http://www.apache.org/licenses/LICENSE-2.0
// 
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.
using System;
using NUnit.Framework;
using Nocandy.LinqResources.Reader;

namespace Nocandy.LinqResources.Reader.Test
{
  /// <summary>
  /// Silly test class to represent user info.
  /// </summary>
  class UserInfo
  {
    public string Name {get; private set;}
    public int Age {get; private set;}
    public UserInfo(string name, int age)
    {
      Name = name;
      Age = age;
    }
  }

  [TestFixture()]
  class TestReader
  {
    private static T Run<T>(Reader<UserInfo, T> it)
    {
      return it(new UserInfo(name: "John Doe", age: 42));
    }

    private static Reader<UserInfo, String> Describe()
    {
      return ui => string.Format("{0}, age {1}", ui.Name, ui.Age);
    }

    [Test()]
    public void SimpleAccess ()
    {
      Assert.AreEqual("John Doe, age 42", Run (Describe()));
    }

    [Test()]
    public void Mapping()
    {
      Assert.AreEqual ("Hello, my name is John Doe, age 42",
                       Run(from d in Describe () select "Hello, my name is " + d));
    }

    private Reader<UserInfo, string> Name()
    {
      return from ui in ReaderOps.Get<UserInfo>() select ui.Name;
    }

    [Test()]
    public void Sequencing()
    {
      Assert.AreEqual (new {desc = "John Doe, age 42", nm = "John Doe"},
                       Run(from desc in Describe () from nm in Name ()
                           select new {desc, nm}));
    }
  }
}

